//
//  RTcmixManager.m
//
//		Singleton (one instance throughout the app) used to mediate between
//		interface and RTcmixPlayer audio system
//
//  Created by Brad Garton on 7/17/09.
//	modified with RTcmixManager by Damon Holzborn 3/2010
//
//	-- basic skeleton for development, usually the methods are called by the AppDelegate
//

#import "RTcmixManager.h"


static RTcmixManager *sharedSingletonManager = nil;

@implementation RTcmixManager

@synthesize rtcmixPlayer;			// the playback audio system (uses AudioQueues)


# pragma mark RTcmix Commands


// add any action methods here associated with interface objects
- (void)allez {
	char *thescore = { "\
		dex = trand(0, linsks) \
		MIX(0, insks[dex], durs[dex], 1, 0, 1) \
	"};

	parse_score(thescore, strlen(thescore));
}

- (void)plus {
	char *thescore = { "\
		dex = trand(0, linsks) \
		transp = irand(-0.12, 0.12) \
		TRANS(0, insks[dex], translen(durs[dex], transp), 0.7, transp) \
	"};

	parse_score(thescore, strlen(thescore));
}

- (void)fin {
	flush_sched();

	char *thescore = { "\
		GVERB(0, 0, 999, 0.5, 178.0, 77.0, 0.71, 0.34, -10.0, -11.0, -1.0, 0.0) \
	"};

	parse_score(thescore, strlen(thescore));
}

- (void) newGain:(Float32)newgain {
	[rtcmixPlayer newGain:newgain];
}

// checkBangMethod is called from within the playbackCallback() callback if check_bang()
// returns 1, set by MAXBANG() in the score.  This needs to be implemented even if not used.
- (void)checkBangMethod { }


- (void)startRTcmix {
	
	// Initialize the player.
	RTcmixPlayer *thePlayer = [[RTcmixPlayer alloc] initPlayer];
//	use this initialization for the RemoteIO AU (flag "0" is for no input)
//	RTcmixPlayer *thePlayer = [[RTcmixPlayer alloc] initPlayerAUWithInputFlag:0];

	
	if (thePlayer) {		
		self.rtcmixPlayer = thePlayer;
		[thePlayer release];	// decrements the retain count for the thePlayer object
	}


// add any other RTcmix initializations here
	char thescore[1024];

	if ([rtcmixPlayer setSoundFile:@"murailfrags.aiff"] != 1)
	NSLog(@"problem setting the soundfile");
	char *spath = (char *)[[rtcmixPlayer soundFilePath] UTF8String];

	sprintf(thescore, "print_off()  srand()  rtinput(\"%s\") \
		insks = { 0.0, 12.05, 18.06, 23.063, 25.0, 25.75, 27.50, 32.445, 45.302, \
			48.672, 55.927, 64.600, 76.283, 94.985, 99.927, 114.385, 125.445, \
			141.900, 146.641, 155.816, 157.744, 160.441, 170.634, 175.529, 178.552, \
			192.870, 198.203, 208.414, 216.027, 231.693, 241.512, 251.759, 259.666 } \
		linsks = len(insks) \
		durs = { 12.05, 6.01, 4.1, 1.0369, 0.742, 1.753, 4.943, 12.858, 3.377, \
			6.976, 8.516, 11.683, 18.704, 4.943, 14.459, 11.061, 16.472, \
			4.742, 8.942, 1.867, 2.695, 10.196, 4.897, 3.026, 14.225, \
			5.333, 10.212, 7.614, 15.666, 9.817, 10.254, 7.911, 11.715 } \
		bus_config(\"TRANS\", \"in 0-1\", \"aux 0 out\") \
		bus_config(\"GVERB\", \"aux 0 in\", \"out 0-1\") \
		GVERB(0, 0, 999, 0.5, 178.0, 77.0, 0.71, 0.34, -10.0, -11.0, -1.0, 0.0) \
			", spath);

	parse_score(thescore, strlen(thescore));

}


#pragma mark -

#pragma mark Singleton Methods
+ (id)sharedManager
{
    @synchronized(self) {
        if (sharedSingletonManager == nil) {
            [[self alloc] init]; // assignment not done here
        }
    }
    return sharedSingletonManager;
}

+ (id)allocWithZone:(NSZone *)zone
{
    @synchronized(self) {
        if (sharedSingletonManager == nil) {
            sharedSingletonManager = [super allocWithZone:zone];
            return sharedSingletonManager;  // assignment and return on first allocation
        }
    }
    return nil; //on subsequent allocation attempts return nil
}

- (id)copyWithZone:(NSZone *)zone
{
    return self;
}

- (id)retain
{
    return self;
}

- (unsigned)retainCount
{
    return UINT_MAX;  //denotes an object that cannot be released
}

- (void)release
{
    //do nothing
}

- (id)autorelease
{
    return self;
}

@end
