//
//  RTcmixPlayer.h
//  "base" class -- use this for iRTcmix audio work
//	will function in both AudioQueueBuffer and AudioUnit mode
//
//  Created by Brad Garton on 7/17/09.
//	modified with RTcmixManager by Damon Holzborn 3/2010
//	additional work done by Brad Garton, 4/2010
//

#include <AudioToolbox/AudioToolbox.h>
#import <UIKit/UIKit.h>


// BGG -- not sure how these should be set.  These I chose by trial-and error to get best performance
// kNumberAudioDataBuffers is only used in AudioQueueBuffer mode
#define kNumberAudioDataBuffers	4
#define NUMPACKETSPERCALLBACK 1024
#define NCHANS 2

// RTcmix stuff, external functions called from the RTcmixPlayer
extern int maxmsp_rtsetparams(float sr, int nchans, int vecsize, float *mm_inbuf, float *mm_outbuf, char *mm_errbuf);
extern void pullTraverse(short *inbuf, short *outbuf); // we pass in pointers to the audio bufs
extern int check_bang();
extern void pfield_set(int inlet, float pval);
extern void buffer_set(char *bufname, float *bufstart, int nframes, int nchans, int modtime);
char errbuf[1024]; // not used right now, but necessary for RTcmix error-writing
extern int rtcmixmain();
extern int parse_score(char *thebuf, int buflen);
extern void flush_sched();


@interface RTcmixPlayer : NSObject {

	AudioQueueBufferRef				buffers[kNumberAudioDataBuffers];	// the audio queue buffers for the audio queue
	UInt32							bufferByteSize;						// the number of bytes to use in each audio queue buffer
	UInt32							numPacketsPerCallback;				// the number of audio data packets to read into each audio queue buffer
	Float32							gain;								// the gain (relative audio level) for the playback audio queue
	BOOL							audioPlayerShouldStopImmediately;	// this is used for 'async' vs. 'sync' shutdown of audio
	AudioQueueRef					queueObject;						// the audio queue object being used for playback
	AudioStreamBasicDescription		audioFormat;						// used in setPlaybackStuff to set up audio characteristics
// AudioUnit stuff
	AURenderCallbackStruct			inputProc;
	AudioUnit						rioUnit;							// the RemoteIO Audio Unit
	short							*rtcmixoutbuf;
	int								outbufdex;
	short							*rtcmixinbuf;
	int								inbufdex;
	int								audiounitflag;						// 1 if running RemoteIO, 0 otherwise
	int								inputflag;							// 1 if RemoteIO input enabled
// soundfile/buffer playback stuff
	NSString						*soundFilePath;						// path to input soundfile
	void							*sampBuf;							// pointer to memory to load audio into
	float							*fsampBuf;							// floating-pt sample buff for RTcmix
}

@property (readwrite) UInt32						numPacketsPerCallback;
@property (readwrite) BOOL							audioPlayerShouldStopImmediately;
@property (readwrite) UInt32						bufferByteSize;
@property (readwrite) Float32						gain;
@property (readwrite) AudioQueueRef					queueObject;
@property (readwrite) AudioStreamBasicDescription	audioFormat;
// AudioUnit stuff
@property (nonatomic, assign)	AURenderCallbackStruct	inputProc;
@property (nonatomic, assign)	AudioUnit				rioUnit;
@property (readwrite)			int						outbufdex;
@property (readwrite)			short					*rtcmixoutbuf;
@property (readwrite)			int						inbufdex;
@property (readwrite)			short					*rtcmixinbuf;
@property (readwrite)			int						audiounitflag;
@property (readwrite)			int						inputflag;
// soundfile/buffer playback stuff
@property (nonatomic,retain) NSString				*soundFilePath;



- (void) setInlet:(int)inlet withVal:(Float32)val;
- (int) setSoundFile: (NSString *)sfname;
- (int) loadSampleBuffer: (NSString *)bufname;
- (id) initPlayer;
- (id) initPlayerAUWithInputFlag:(int)iflag;	// init RTcmixPlayer using RemoteIO AudioUnit
- (void) setPlaybackStuff;
- (int) setupRemoteIO;							// for AudioUnit
- (void) setupPlaybackAudioQueueObject;
- (void) setupAudioQueueBuffers;
- (void) newGain: (Float32)newgain;
- (void) play;
- (void) stop;
- (void) pause;
- (void) resume;

@end
